# Copyright (c) 2016, Camptocamp SA
# All rights reserved.

# Auto-generated file. Do not Edit!
INFO = {
    "git_hash": "54cb1f3b9479f36805596df042d8f63286d76a54",
    "version": "2.1",
    "git_tag": "2.1.0.dev23",
}
