OpenLayers.Util.extend(OpenLayers.Lang.en, {
    "layertree": "Themes",
    "blank": "Blank",
    "density": "Population density (hab./km²)",

    /* print */
    "title": "Title",
    "comment": "Description",
    "debug": "Add debug header"
});
